package com.microsoft.trouterclient;

import java.util.*;

class TrouterRequest implements ITrouterRequest {
    @Override
    public String getMethod() {
        return method;
    }

    @Override
    public String getUrlPathComponent() {
        return urlPathComponent;
    }

    @Override
    public List<Map.Entry<String, String>> getHeaders() {
        return headers;
    }

    @Override
    public String getBody() {
        return body;
    }

    TrouterRequest(String method, String urlPathComponent, List<Map.Entry<String, String>> headers, String body) {
        this.method           = method;
        this.urlPathComponent = urlPathComponent;
        this.headers          = Collections.unmodifiableList(headers);
        this.body             = body;
    }

    private final String method;
    private final String urlPathComponent;
    private final List<Map.Entry<String, String>> headers;
    private final String body;
}

class TrouterResponse implements ITrouterResponse {
    @Override
    public long getId() {
        return id;
    }

    @Override
    public void setStatus(int statusCode) {
        this.statusCode = statusCode;
    }

    @Override
    public int getStatusCode() {
        return statusCode;
    }

    @Override
    public void setHeader(String header, String value) {
        headers.add(new AbstractMap.SimpleEntry<String, String>(header, value));
    }

    @Override
    public List<Map.Entry<String, String>> getHeaders() {
        return headers;
    }

    @Override
    public void setBody(String data) {
        body = data;
    }

    @Override
    public String getBody() {
        return body;
    }

    @Override
    public native MessageSendStatusCode send();

    TrouterResponse(long id) {
        this.id         = id;
        this.headers    = new ArrayList<Map.Entry<String, String>>();
        this.statusCode = -1;
        this.body       = "";
    }

    private final long id;
    private final List<Map.Entry<String, String>> headers;
    private int statusCode;
    private String body;
}

class TrouterConnectionInfo implements ITrouterConnectionInfo {
    @Override
    public String getBaseEndpointUrl() {
        return baseEndpointUrl;
    }

    @Override
    public boolean isNewEndpointUrl() {
        return newEndpointUrl;
    }

    @Override
    public String getC2cUrlBase() {
        return c2cUrlBase;
    }

    @Override
    public String getClientId() {
        return clientId;
    }

    @Override
    public String getConnectionId() {
        return connectionId;
    }

    @Override
    public int getConnectionTtlSec() {
        return connectionTtlSec;
    }

    TrouterConnectionInfo(String baseEndpointUrl, boolean newEndpointUrl, String c2cUrlBase, String clientId, String connectionId, int connectionTtlSec) {
        this.baseEndpointUrl  = baseEndpointUrl;
        this.newEndpointUrl   = newEndpointUrl;
        this.c2cUrlBase       = c2cUrlBase;
        this.clientId         = clientId;
        this.connectionId     = connectionId;
        this.connectionTtlSec = connectionTtlSec;
    }

    private final String baseEndpointUrl;
    private final boolean newEndpointUrl;
    private final String c2cUrlBase;
    private final String clientId;
    private final String connectionId;
    private final int connectionTtlSec;
}

class Trouter extends ITrouter {
    public Trouter(long nativeTrouterPtr) {
        if (nativeTrouterPtr == 0) {
            throw new IllegalArgumentException("nativeTrouterPtr cannot be 0 (null)");
        }
        this.nativeTrouterPtr = nativeTrouterPtr;
    }

    public native boolean registerListener(ITrouterListener listener, String path, String loggingName);
    public native boolean unregisterListener(ITrouterListener listener);

    private static native void nativeInit();

    static {
        nativeInit();
    }

    private long nativeTrouterPtr;
}
